/*
 * Grdc - GTK+/Gnome Remote Desktop Client
 * Copyright (C) 2009 - Vic Lee 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */

#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <stdlib.h>
#include "config.h"
#include "grdcpublic.h"
#include "grdcpixmaps.h"
#include "grdcmain.h"
#include "grdcfilemanager.h"
#include "grdcfileeditor.h"
#include "grdcconnectionwindow.h"
#include "grdcpref.h"
#include "grdcprefdialog.h"

#ifdef HAVE_LIBGCRYPT
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#include <pthread.h>
#include <gcrypt.h>
GCRY_THREAD_OPTION_PTHREAD_IMPL;
#endif

int
main (int argc, char* argv[])
{
    gint ret;

    bindtextdomain (GETTEXT_PACKAGE, GRDC_LOCALEDIR);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
    textdomain (GETTEXT_PACKAGE);

#ifdef HAVE_PTHREAD
    g_thread_init (NULL);
    gdk_threads_init ();
#endif

#ifdef HAVE_LIBGCRYPT
    gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
#endif

    THREADS_ENTER

    gtk_init (&argc, &argv);

    grdc_file_manager_init ();
    grdc_pref_init ();
    grdc_pixmap_init ();

    if (argc >= 2 && g_strcmp0 (argv[1], "-p") == 0)
    {
        grdc_pref_dialog_run (NULL, (argc >= 3 ? atoi (argv[2]) : 0));
        return 0;
    }
    else if (argc >= 2 && g_strcmp0 (argv[1], "-q") == 0)
    {
        ret = grdc_file_editor_open_temp (NULL, TRUE);
        if (ret != GTK_RESPONSE_OK) return 0;
    }
    else if (argc >= 2 && g_strcmp0 (argv[1], "-n") == 0)
    {
        ret = grdc_file_editor_create (NULL, TRUE);
        if (ret != GTK_RESPONSE_OK) return 0;
    }
    else if (argc >= 3 && g_strcmp0 (argv[1], "-c") == 0)
    {
        if (!grdc_connection_window_open (NULL, argv[2], TRUE)) return 0;
    }
    else if (argc >= 3 && g_strcmp0 (argv[1], "-e") == 0)
    {
        ret = grdc_file_editor_open (NULL, argv[2], TRUE);
        if (ret != GTK_RESPONSE_OK) return 0;
    }
    else
    {
        grdc_main_open ();
    }
    
    gtk_main ();

    THREADS_LEAVE    

    return 0;
}

