/*
 * Grdc - GTK+/Gnome Remote Desktop Client
 * Copyright (C) 2009 - Vic Lee 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "grdcstringarray.h"
#include "grdcpref.h"

const gchar *default_resolutions = "640x480,800x600,1024x768,1152x864,1280x960,1400x1050";

gchar *grdc_pref_file;

GrdcPref grdc_pref;

void
grdc_pref_init (void)
{
    GKeyFile *gkeyfile;

    grdc_pref_file = g_strdup_printf ("%s/.grdc/grdc.pref", g_get_home_dir ());
    gkeyfile = g_key_file_new ();
    g_key_file_load_from_file (gkeyfile, grdc_pref_file, G_KEY_FILE_NONE, NULL);

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "separated_process", NULL))
        grdc_pref.separated_process = g_key_file_get_boolean (gkeyfile, "grdc_pref", "separated_process", NULL);
    else
        grdc_pref.separated_process = FALSE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "save_view_mode", NULL))
        grdc_pref.save_view_mode = g_key_file_get_boolean (gkeyfile, "grdc_pref", "save_view_mode", NULL);
    else
        grdc_pref.save_view_mode = TRUE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "invisible_toolbar", NULL))
        grdc_pref.invisible_toolbar = g_key_file_get_boolean (gkeyfile, "grdc_pref", "invisible_toolbar", NULL);
    else
        grdc_pref.invisible_toolbar = FALSE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "default_action", NULL))
        grdc_pref.default_action = g_key_file_get_integer (gkeyfile, "grdc_pref", "default_action", NULL);
    else
        grdc_pref.default_action = GRDC_ACTION_CONNECT;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "scale_quality", NULL))
        grdc_pref.scale_quality = g_key_file_get_integer (gkeyfile, "grdc_pref", "scale_quality", NULL);
    else
        grdc_pref.scale_quality = GDK_INTERP_HYPER;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "groups", NULL))
        grdc_pref.groups = g_key_file_get_string (gkeyfile, "grdc_pref", "groups", NULL);
    else
        grdc_pref.groups = g_strdup ("");

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "hide_toolbar", NULL))
        grdc_pref.hide_toolbar = g_key_file_get_boolean (gkeyfile, "grdc_pref", "hide_toolbar", NULL);
    else
        grdc_pref.hide_toolbar = FALSE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "hide_statusbar", NULL))
        grdc_pref.hide_statusbar = g_key_file_get_boolean (gkeyfile, "grdc_pref", "hide_statusbar", NULL);
    else
        grdc_pref.hide_statusbar = FALSE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "small_toolbutton", NULL))
        grdc_pref.small_toolbutton = g_key_file_get_boolean (gkeyfile, "grdc_pref", "small_toolbutton", NULL);
    else
        grdc_pref.small_toolbutton = FALSE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "view_file_mode", NULL))
        grdc_pref.view_file_mode = g_key_file_get_integer (gkeyfile, "grdc_pref", "view_file_mode", NULL);
    else
        grdc_pref.view_file_mode = GRDC_VIEW_FILE_LIST;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "resolutions", NULL))
        grdc_pref.resolutions = g_key_file_get_string (gkeyfile, "grdc_pref", "resolutions", NULL);
    else
        grdc_pref.resolutions = g_strdup (default_resolutions);

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "main_width", NULL))
        grdc_pref.main_width = MAX (600, g_key_file_get_integer (gkeyfile, "grdc_pref", "main_width", NULL));
    else
        grdc_pref.main_width = 600;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "main_height", NULL))
        grdc_pref.main_height = MAX (400, g_key_file_get_integer (gkeyfile, "grdc_pref", "main_height", NULL));
    else
        grdc_pref.main_height = 400;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "main_maximize", NULL))
        grdc_pref.main_maximize = g_key_file_get_boolean (gkeyfile, "grdc_pref", "main_maximize", NULL);
    else
        grdc_pref.main_maximize = FALSE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "sshtunnel_port", NULL))
        grdc_pref.sshtunnel_port = g_key_file_get_integer (gkeyfile, "grdc_pref", "sshtunnel_port", NULL);
    else
        grdc_pref.sshtunnel_port = DEFAULT_SSHTUNNEL_PORT;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "applet_quick_ontop", NULL))
        grdc_pref.applet_quick_ontop = g_key_file_get_boolean (gkeyfile, "grdc_pref", "applet_quick_ontop", NULL);
    else
        grdc_pref.applet_quick_ontop = FALSE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "applet_hide_count", NULL))
        grdc_pref.applet_hide_count = g_key_file_get_boolean (gkeyfile, "grdc_pref", "applet_hide_count", NULL);
    else
        grdc_pref.applet_hide_count = FALSE;

    if (g_key_file_has_key (gkeyfile, "grdc_pref", "recent_maximum", NULL))
        grdc_pref.recent_maximum = g_key_file_get_integer (gkeyfile, "grdc_pref", "recent_maximum", NULL);
    else
        grdc_pref.recent_maximum = 10;

    g_key_file_free (gkeyfile);
}

void
grdc_pref_save (void)
{
    GKeyFile *gkeyfile;
    gchar *content;
    gsize length;

    gkeyfile = g_key_file_new ();

    g_key_file_load_from_file (gkeyfile, grdc_pref_file, G_KEY_FILE_NONE, NULL);

    g_key_file_set_boolean (gkeyfile, "grdc_pref", "separated_process", grdc_pref.separated_process);
    g_key_file_set_boolean (gkeyfile, "grdc_pref", "save_view_mode", grdc_pref.save_view_mode);
    g_key_file_set_boolean (gkeyfile, "grdc_pref", "invisible_toolbar", grdc_pref.invisible_toolbar);
    g_key_file_set_integer (gkeyfile, "grdc_pref", "default_action", grdc_pref.default_action);
    g_key_file_set_integer (gkeyfile, "grdc_pref", "scale_quality", grdc_pref.scale_quality);
    g_key_file_set_string (gkeyfile, "grdc_pref", "groups", grdc_pref.groups);
    g_key_file_set_boolean (gkeyfile, "grdc_pref", "hide_toolbar", grdc_pref.hide_toolbar);
    g_key_file_set_boolean (gkeyfile, "grdc_pref", "hide_statusbar", grdc_pref.hide_statusbar);
    g_key_file_set_boolean (gkeyfile, "grdc_pref", "small_toolbutton", grdc_pref.small_toolbutton);
    g_key_file_set_integer (gkeyfile, "grdc_pref", "view_file_mode", grdc_pref.view_file_mode);
    g_key_file_set_string (gkeyfile, "grdc_pref", "resolutions", grdc_pref.resolutions);
    g_key_file_set_integer (gkeyfile, "grdc_pref", "main_width", grdc_pref.main_width);
    g_key_file_set_integer (gkeyfile, "grdc_pref", "main_height", grdc_pref.main_height);
    g_key_file_set_boolean (gkeyfile, "grdc_pref", "main_maximize", grdc_pref.main_maximize);
    g_key_file_set_integer (gkeyfile, "grdc_pref", "sshtunnel_port", grdc_pref.sshtunnel_port);
    g_key_file_set_boolean (gkeyfile, "grdc_pref", "applet_quick_ontop", grdc_pref.applet_quick_ontop);
    g_key_file_set_boolean (gkeyfile, "grdc_pref", "applet_hide_count", grdc_pref.applet_hide_count);
    g_key_file_set_integer (gkeyfile, "grdc_pref", "recent_maximum", grdc_pref.recent_maximum);

    content = g_key_file_to_data (gkeyfile, &length, NULL);
    g_file_set_contents (grdc_pref_file, content, length, NULL);

    g_key_file_free (gkeyfile);
    g_free (content);
}

void
grdc_pref_add_recent (const gchar *protocol, const gchar *server)
{
    GrdcStringArray *array;
    GKeyFile *gkeyfile;
    gchar key[20];
    gchar *val;
    gchar *content;
    gsize length;

    if (grdc_pref.recent_maximum <= 0 || server == NULL || server[0] == 0) return;

    /* Load original value into memory */
    gkeyfile = g_key_file_new ();

    g_key_file_load_from_file (gkeyfile, grdc_pref_file, G_KEY_FILE_NONE, NULL);

    g_snprintf (key, sizeof (key), "recent_%s", protocol);
    array = grdc_string_array_new_from_allocated_string (
        g_key_file_get_string (gkeyfile, "grdc_pref", key, NULL));

    /* Add the new value */
    grdc_string_array_remove (array, server);
    while (array->len >= grdc_pref.recent_maximum)
    {
        grdc_string_array_remove_index (array, 0);
    }
    grdc_string_array_add (array, server);

    /* Save */
    val = grdc_string_array_to_string (array);
    g_key_file_set_string (gkeyfile, "grdc_pref", key, val);
    g_free (val);

    content = g_key_file_to_data (gkeyfile, &length, NULL);
    g_file_set_contents (grdc_pref_file, content, length, NULL);

    g_key_file_free (gkeyfile);
    g_free (content);
}

gchar*
grdc_pref_get_recent (const gchar *protocol)
{
    GKeyFile *gkeyfile;
    gchar key[20];
    gchar *val;

    gkeyfile = g_key_file_new ();

    g_key_file_load_from_file (gkeyfile, grdc_pref_file, G_KEY_FILE_NONE, NULL);

    g_snprintf (key, sizeof (key), "recent_%s", protocol);
    val = g_key_file_get_string (gkeyfile, "grdc_pref", key, NULL);

    g_key_file_free (gkeyfile);

    return val;
}

void
grdc_pref_clear_recent (void)
{
    GKeyFile *gkeyfile;
    gchar *content;
    gsize length;

    gkeyfile = g_key_file_new ();

    g_key_file_load_from_file (gkeyfile, grdc_pref_file, G_KEY_FILE_NONE, NULL);
    g_key_file_set_string (gkeyfile, "grdc_pref", "recent_RDP", "");
    g_key_file_set_string (gkeyfile, "grdc_pref", "recent_VNC", "");

    content = g_key_file_to_data (gkeyfile, &length, NULL);
    g_file_set_contents (grdc_pref_file, content, length, NULL);

    g_key_file_free (gkeyfile);
    g_free (content);
}

