/*
 * Grdc - GTK+/Gnome Remote Desktop Client
 * Copyright (C) 2009 - Vic Lee 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 

#ifndef __GRDCSSH_H__
#define __GRDCSSH_H__

#include "config.h"

#ifdef HAVE_LIBSSH

#include <libssh/libssh.h>
#include <pthread.h>

G_BEGIN_DECLS

typedef struct _GrdcSSHTunnel
{
    SSH_SESSION *session;
    CHANNEL *channel;
    gboolean authenticated;

    gint tunnel_server_sock;
    gint tunnel_client_sock;

    pthread_t thread;
    gboolean running;

    gchar *dest_host;
    gint dest_port;

    gchar *buffer;
    gint buffer_len;

    gchar *error;
} GrdcSSHTunnel;

gchar* grdc_ssh_identity_path (const gchar *id);

/* Auto-detect commonly used private key identities */
gchar* grdc_ssh_find_identity (void);

/* Create a new SSH Tunnel session and connects to the SSH server */
GrdcSSHTunnel* grdc_ssh_tunnel_new (const gchar *server, const gchar *user);

/* Authenticate using password */
gboolean grdc_ssh_tunnel_auth_password (GrdcSSHTunnel* tunnel, const gchar *password);

/* Authenticate using public key */
/* -1: Require passphrase; 0: Failed; 1: Succeeded */
gint grdc_ssh_tunnel_auth_pubkey (GrdcSSHTunnel* tunnel, const gchar *pubkeyfile, 
    const gchar *privkeyfile, const gchar *passphrase);

/* Start the tunnel. A new thread will be started and listening on a local port.
 * dest: The host:port of the remote destination
 * local_port: The listening local port for the tunnel
 */
gboolean grdc_ssh_tunnel_start (GrdcSSHTunnel* tunnel, const gchar *dest, gint local_port);

/* Tells if the tunnel is terminated after start */
gboolean grdc_ssh_tunnel_terminated (GrdcSSHTunnel* tunnel);

/* Free the tunnel */
void grdc_ssh_tunnel_free (GrdcSSHTunnel* tunnel);

G_END_DECLS

#else

#define GrdcSSHTunnel void

#endif /* HAVE_LIBSSH */

#endif  /* __GRDCSSH_H__  */

